<?php
/*
 * ----------------------------------------------------------------------------
 * Va-Database.inc.php                                               05/13/2004
 * This script contains a set of functions that may be useful when dealing with
 * Virtual Advanced database groups, databases, and messages.
 * ----------------------------------------------------------------------------
 * Copyright (C) 2002-2004 Steve Winn. All Rights Reserved.
 *
 * The source code contains a fair amount of comments, which may aid anyone
 * attempting to follow the program or to modify it.
 *
 * For licensing and copyright information, view LICENSE.TXT.
 * ----------------------------------------------------------------------------
 */

require_once('include_dir.php');                    // Get include directory.
require_once($includeDir . 'va-functions.inc.php'); // A VADV-PHP script.


/**
* Va-Database.inc.php :: FindTopicInfo()
*
* This function will retrieve the variables for the selected database topic using
* the passed file string. This function is identical to the GetTopicInfo function
* except you MUST pass the file string to it. This is useful if you are going to
* search the topics multiple times.
*
* @param [type] $topicNo
* @param [type] $fileString
* @param string $type
* @return
*/
function FindTopicInfo($topicNo, $fileString, $type = '') {
    if ((!is_int($topicNo)) && (!is_string($topicNo))) {
        trigger_error(VA_Error(1));
    }
    if (!is_string($fileString)) {
        trigger_error(VA_Error(1));
    }
    $topicNo = intval($topicNo);
    if ($topicNo < 1) {
        $topicNo = 1;
    }

    global $VA_SYSPATHS;                    // Include global variable.

    $type  = trim(strtoupper($type));       // Change the type to uppercase.
    $topic = chr($topicNo + 64);            // Change the topic number into the topic letter.

    $topicInfo = FALSE;
    $position  = 2;                         // Set the starting position.

    for ($x = 0; $position < strlen($fileString); $x++) {
        $dbType = substr($fileString, $position - 2, 1);
        if (($type == '') || (($type != '') && ($type == $dbType))) {
            $string = trim(strtoupper(substr($fileString, $position - 1, 1)));
            if ($string == $topic) {
                $offset = 128 * $x;
                $topicInfo['topicno'] = $topicNo;
                $topicInfo['topic']   = $topic;
                $topicInfo['dbtype']  = GetString($fileString, 1 + $offset, 1);
                $topicInfo['topic']   = GetString($fileString, 2 + $offset, 1);
                $topicInfo['desc']    = GetString($fileString, 3 + $offset, 65);
                $topicInfo['marker1'] = GetLongInt($fileString, 70 + $offset);
                $topicInfo['marker2'] = GetLongInt($fileString, 74 + $offset);
                $topicInfo['dbno']    = GetDBCount($topicNo);
                break;
            }
        }
        $position = $position + 128;
    }

    return $topicInfo;                      // Return the topic information.
}


/**
* Va-Database.inc.php :: GetTopicInfo()
*
* This function will retrieve the variables for the selected database topic.
*
* @param [type] $topicNo
* @param string $type
* @return
*/
function GetTopicInfo($topicNo, $type = '') {
    if ((!is_int($topicNo)) && (!is_string($topicNo))) {
        trigger_error(VA_Error(1));
    }
    $topicNo = intval($topicNo);
    if ($topicNo < 1) {
        $topicNo = 1;
    }

    global $VA_SYSPATHS;                    // Include global variable.
    
    // Set the filename.
    $file = $VA_SYSPATHS['main'] . '/dbgroup.cfg';
    if (!is_file($file)) {
        return FALSE;
    }

    $fileString = ReadCfgFile($file);       // Read the file.

    $type  = trim(strtoupper($type));       // Change the type to uppercase.
    $topic = chr($topicNo + 64);            // Change the topic number into the topic letter.

    $topicInfo = FALSE;
    $position  = 2;                         // Set the starting position.

    for ($x = 0; $position < strlen($fileString); $x++) {
        $dbType = substr($fileString, $position - 2, 1);
        if (($type == '') || (($type != '') && ($type == $dbType))) {
            $string = trim(strtoupper(substr($fileString, $position - 1, 1)));
            if ($string == $topic) {
                $offset = 128 * $x;
                $topicInfo['topicno'] = $topicNo;
                $topicInfo['topic']   = $topic;
                $topicInfo['dbtype']  = GetString($fileString, 1 + $offset, 1);
                $topicInfo['topic']   = GetString($fileString, 2 + $offset, 1);
                $topicInfo['desc']    = GetString($fileString, 3 + $offset, 65);
                $topicInfo['marker1'] = GetLongInt($fileString, 70 + $offset);
                $topicInfo['marker2'] = GetLongInt($fileString, 74 + $offset);
                $topicInfo['dbno']    = GetDBCount($topicNo);
                break;
            }
        }
        $position = $position + 128;
    }

    return $topicInfo;                      // Return the topic information.
}


/**
* Va-Database.inc.php :: GetTopics()
*
* This function will retrieve the topic numbers and store them into an array.
*
* @param string $type
* @return
*/
function GetTopics($type = '') {
    global $VA_SYSPATHS;                    // Include the global variable.

    // Set the filename.
    $file = $VA_SYSPATHS['main'] . '/dbgroup.cfg';
    if (!is_file($file)) {
        return FALSE;
    }

    $fileString = ReadCfgFile($file);       // Read the file.

    $type = trim(strtoupper($type));        // Change the topic type into uppercase.

    $position  = 2;                         // Set the starting position.
    $topics[0] = 0;

    for ($x = 0; $position < strlen($fileString); $x++) {
        $string = trim(strtoupper(substr($fileString, $position - 2, 1)));
        if ((($type == '') || (($type != '') && ($type == $string))) && ($string != '')) {
            $group = trim(strtoupper(substr($fileString, $position - 1, 1)));
            $group = ord($group) - 64;
            if (GetDbCount($group) > 0) {
                array_push($topics, $group);
            }
        }
        $position = $position + 128;
    }

    $topics[0] = count($topics) - 1;
    
    return $topics;                        // Return the topic numbers.
}


/**
* Va-Database.inc.php :: GetTopicCount()
*
* This function will determine the total number of topics and return that value.
*
* @param string $type
* @return
*/
function GetTopicCount($type = '') {
    global $VA_SYSPATHS;                    // Include the global variable.

    // Set the filename.
    $file = $VA_SYSPATHS['main'] . '/dbgroup.cfg';
    if (!is_file($file)) {
        return FALSE;
    }

    $fileString = ReadCfgFile($file);       // Read the file.

    $type = trim(strtoupper($type));        // Change the topic type into uppercase.

    $position   = 2;                        // Set the starting position.
    $topicNo    = 0;

    for ($x = 0; $position < strlen($fileString); $x++) {
        $string = trim(strtoupper(substr($fileString, $position - 2, 1)));
        if ((($type == '') || (($type != '') && ($type == $string))) && ($string != '')) {
            $group = trim(strtoupper(substr($fileString, $position - 1, 1)));
            $group = ord($group) - 64;
            if (GetDbCount($group) > 0) {
                $topicNo++;
            }
        }
        $position = $position + 128;
    }

    return $topicNo;                        // Return the topic count.
}


/**
* Va-Database.inc.php :: FindDbInfo()
*
* This function will retrieve the variables for the selected database from the
* passed fileString. This function is identical to GetDbInfo except you MUST
* pass the fileString. This function is more useful if you are going to be
* searching the same database multiple times.
*
* @param [type] $topicNo
* @param [type] $db
* @param [type] $fileString
* @return
*/
function FindDbInfo($topicNo, $db, $fileString) {
    if ((!is_int($topicNo)) && (!is_string($topicNo))) {
        trigger_error(VA_Error(1));
    }
    if ((!is_int($db)) && (!is_string($db))) {
        trigger_error(VA_Error(1));
    }
    if (!is_string($fileString)) {
        trigger_error(VA_Error(1));
    }
    $topicNo = intval($topicNo);
    $db = intval($db);
    if ($topicNo < 1) {
        $topicNo = 1;
    }
    if ($db < 1) {
        $db = 1;
    }

    $topic = chr($topicNo + 64);            // Convert the topic number into the topic letter.

    global $VA_SYSPATHS;                    // Include the global variable.

    $dbInfo   = array();
    $position = 75;
    $num      = 0;

    // Set the message number to zero.
    $dbInfo['msgno'] = 0;

    for ($x = 0; $position < strlen($fileString); $x++) {
        $string   = trim(strtoupper(substr($fileString, $position - 1, 1)));
        $position = $position + 131;
        if ($string == $topic) {
            $num++;
            if ($num == $db) {
                $offset = 131 * $x;
                $dbInfo['topicno']    = $topicNo;
                $dbInfo['topic']      = $topic;
                $dbInfo['dbno']       = $db;
                $dbInfo['filename']   = GetString($fileString, 1 + $offset, 8);
                $dbInfo['filepath']   = GetString($fileString, 9 + $offset, 32);
                $dbInfo['desc']       = GetString($fileString, 41 + $offset, 32);
                $dbInfo['private']    = GetInteger($fileString, 73 + $offset);
                $dbInfo['dbgroup']    = GetString($fileString, 75 + $offset, 1);
                $dbInfo['maxmsg']     = GetInteger($fileString, 76 + $offset);
                $dbInfo['minreadsl']  = GetInteger($fileString, 78 + $offset);
                $dbInfo['minwritesl'] = GetInteger($fileString, 80 + $offset);
                $dbInfo['agelimit']   = GetInteger($fileString, 82 + $offset);
                $dbInfo['accflags']   = GetAccFlags(GetLongInt($fileString, 84 + $offset));
                $dbInfo['tagline']    = GetInteger($fileString, 88 + $offset);
                $dbInfo['dispname']   = GetInteger($fileString, 90 + $offset);
                $dbInfo['randtitle']  = GetInteger($fileString, 92 + $offset);
                $dbInfo['qwkconfno']  = GetInteger($fileString, 94 + $offset);
                $dbInfo['moderator']  = GetInteger($fileString, 96 + $offset);
                $dbInfo['validate']   = GetInteger($fileString, 98 + $offset);
                $dbInfo['freqable']   = GetInteger($fileString, 100 + $offset);
                $dbInfo['devicetype'] = GetInteger($fileString, 102 + $offset);
                $dbInfo['dbtype']     = GetString($fileString, 104 + $offset, 1);
                $dbInfo['msgno']      = GetMessageCount($dbInfo);
                if ($dbInfo['msgno'] == '') $dbInfo['msgno'] = 0;
                break;
            }
        }
    }

    return $dbInfo;                         // Return the database information.
}


/**
* Va-Database.inc.php :: GetDbInfo()
*
* This function will retrieve the variables for the selected database.
*
* @param [type] $topicNo
* @param [type] $db
* @return
*/
function GetDbInfo($topicNo, $db) {
    if ((!is_int($topicNo)) && (!is_string($topicNo))) {
        trigger_error(VA_Error(1));
    }
    if ((!is_int($db)) && (!is_string($db))) {
        trigger_error(VA_Error(1));
    }
    $topicNo = intval($topicNo);
    $db = intval($db);
    if ($topicNo < 1) {
        $topicNo = 1;
    }
    if ($db < 1) {
        $db = 1;
    }

    $topic = chr($topicNo + 64);            // Convert the topic number into the topic letter.

    global $VA_SYSPATHS;                    // Include the global variable.

    // Set the filename.
    $file = $VA_SYSPATHS['main'] . '/database.cfg';
    if (!is_file($file)) {
        return FALSE;
    }

    $fileString = ReadCfgFile($file);       // Read the file.

    $dbInfo   = array();
    $position = 75;
    $num      = 0;

    // Set message number to zero.
    $dbInfo['msgno'] = 0;

    for ($x = 0; $position < strlen($fileString); $x++) {
        $string   = trim(strtoupper(substr($fileString, $position - 1, 1)));
        $position = $position + 131;
        if ($string == $topic) {
            $num++;
            if ($num == $db) {
                $offset = 131 * $x;
                $dbInfo['topicno']    = $topicNo;
                $dbInfo['topic']      = $topic;
                $dbInfo['dbno']       = $db;
                $dbInfo['filename']   = GetString($fileString, 1 + $offset, 8);
                $dbInfo['filepath']   = GetString($fileString, 9 + $offset, 32);
                $dbInfo['desc']       = GetString($fileString, 41 + $offset, 32);
                $dbInfo['private']    = GetInteger($fileString, 73 + $offset);
                $dbInfo['dbgroup']    = GetString($fileString, 75 + $offset, 1);
                $dbInfo['maxmsg']     = GetInteger($fileString, 76 + $offset);
                $dbInfo['minreadsl']  = GetInteger($fileString, 78 + $offset);
                $dbInfo['minwritesl'] = GetInteger($fileString, 80 + $offset);
                $dbInfo['agelimit']   = GetInteger($fileString, 82 + $offset);
                $dbInfo['accflags']   = GetAccFlags(GetLongInt($fileString, 84 + $offset));
                $dbInfo['tagline']    = GetInteger($fileString, 88 + $offset);
                $dbInfo['dispname']   = GetInteger($fileString, 90 + $offset);
                $dbInfo['randtitle']  = GetInteger($fileString, 92 + $offset);
                $dbInfo['qwkconfno']  = GetInteger($fileString, 94 + $offset);
                $dbInfo['moderator']  = GetInteger($fileString, 96 + $offset);
                $dbInfo['validate']   = GetInteger($fileString, 98 + $offset);
                $dbInfo['freqable']   = GetInteger($fileString, 100 + $offset);
                $dbInfo['devicetype'] = GetInteger($fileString, 102 + $offset);
                $dbInfo['dbtype']     = GetString($fileString, 104 + $offset, 1);
                $dbInfo['msgno']      = GetMessageCount($dbInfo);
                if ($dbInfo['msgno'] == '') $dbInfo['msgno'] = 0;
                break;
            }
        }
    }

    return $dbInfo;                         // Return the database information.
}


/**
* Va-Database.inc.php :: GetDbCount()
*
* This function will get the number of available databases in the selected
* topic. It will filter out those which the user does not have access to.
*
* @param [type] $topicNo
* @return
*/
function GetDbCount($topicNo) {
    if ((!is_int($topicNo)) && (!is_string($topicNo))) {
        trigger_error(VA_Error(1));
    }
    $topicNo = intval($topicNo);
    if ($topicNo < 0) {
        trigger_error(VA_Error(2));
    }

    $topic = chr($topicNo + 64);

    global $VA_SYSPATHS;                    // Include the global variable.

    $file = $VA_SYSPATHS['main'] . '/database.cfg';
    if (!is_file($file)) {
        return FALSE;
    }
    
    $fileString = ReadCfgFile($file);       // Read the file.

    $position   = 75;                       // Set the starting position.
    $dbNo       = 0;

    for ($x = 0; $position < strlen($fileString); $x++) {
        $string = trim(strtoupper(substr($fileString, $position - 1, 1)));
        $position = $position + 131;
        if ($string == $topic) {
            $offset = 131 * $x;
            $dbInfo['minreadsl'] = GetInteger($fileString, 78 + $offset);
            $dbInfo['agelimit']  = GetInteger($fileString, 82 + $offset);
            $dbInfo['accflags']  = GetAccFlags(GetLongInt($fileString, 84 + $offset));
            if (DbAccess($dbInfo)) {
                $dbNo++;
            }
        }
    }

    return $dbNo;                           // Return the database count.
}


/**
* Va-Database.inc.php :: GetMessagePointer()
*
* This function will return the specified user's message pointer for the
* current database.
*
* @param [type] $dbInfo
* @param [type] $userNo
* @return
*/
function GetMessagePointer($dbInfo, $userNo) {
    if (!is_array($dbInfo)) {
        trigger_error(VA_Error(1));
    }
    if ((!is_int($userNo)) && (!is_string($userNo))) {
        trigger_error(VA_Error(1));
    }
    $userNo = intval($userNo);
    if ($userNo < 1) {
        return 32760;
    }
    
    global $VA_SYSPATHS;                    // Include the global variable.
    
    $file = $VA_SYSPATHS['db'] . '/' . $dbInfo['filename'] . '.QSC';
    if (!is_file($file)) {
        return 0;
    }

    $fileString = ReadCfgFile($file);
    $offset = ($userNo - 1) * 4;
    $pointer = GetLongInt($fileString, 1 + $offset);
    
    return $pointer;                        // Return the message pointer.
}


/**
* Va-Database.inc.php :: WriteMessagePointer()
*
* This function will update the message pointer file for the specified database
* with either the supplied message number, or if no message number is passed,
* then it will increase the pointer by one.
*
* @param [type] $dbInfo
* @param [type] $userNo
* @param integer $msgNo
* @return
*/
function WriteMessagePointer($dbInfo, $userNo, $msgNo = 0) {
    if (!is_array($dbInfo)) {
        trigger_error(VA_Error(1));
    }
    if ((!is_int($userNo)) && (!is_string($userNo))) {
        trigger_error(VA_Error(1));
    }
    $userNo = intval($userNo);
    if ($userNo < 1) {
        return 32760;
    }

    global $VA_SYSPATHS;                    // Include the global variable.

    $file = $VA_SYSPATHS['db'] . '/' . $dbInfo['filename'] . '.QSC';
    if (!is_file($file)) {
        $fileString = str_pad('', $userNo * 4, chr(0));
    } else {
        $fileString = ReadCfgFile($file);
    }
    $offset = ($userNo - 1) * 4;
    $pointer = GetLongInt($fileString, 1 + $offset);
    
    if ($msgNo != 0) {
        $newPointer = $msgNo;
    } else {
        $newPointer = $pointer + 1;
    }

    $fileString = PutLongInt($fileString, $newPointer, 1 + $offset);
    $status = WriteCfgFile($file, $fileString);
    if (!$status) {
        return $status;
    }

    return $pointer;                        // Return the message pointer.
}


/**
* Va-Database.inc.php :: UpdateMessagePointers()
*
* This function will scan ALL message databases and update ALL message pointers
* to the most current message. If $ignore is set to 1, it will obey the
* join/ignore flags set by the user.
*
* @param [type] $userNo
* @param integer $ignore
* @return
*/
function UpdateMessagePointers($userNo, $ignore = 1) {
    if ((!is_int($userNo)) && (!is_string($userNo))) {
        trigger_error(VA_Error(1));
    }
    $userNo = intval($userNo);

    global $VA_SYSPATHS;
    global $includeDir;
    include_once($includeDir . 'va-userfile.inc.php');
    
    $user = GetUser($userNo);
    $dbString = ReadCfgFile($VA_SYSPATHS['main'] . '/database.cfg');

    $topics = GetTopics('M');               // Get the accessible topics.
    for ($x = 1; $x <= $topics[0]; $x++) {
        $dbno = GetDbCount($topics[$x]);
        for ($y = 1; $y <= $dbno; $y++) {
            $dbInfo = FindDbInfo($topics[$x], $y, $dbString);
            if ($ignore == 0) {
                if (DbAccess($dbInfo, '', $user)) {
                    WriteMessagePointer($dbInfo, $userNo, $dbInfo['msgno']);
                }
            } else {
                if ((GetMessagePointer($dbInfo, $userNo) < 32760) &&
                    (DbAccess($dbInfo, '', $user))) {
                    WriteMessagePointer($dbInfo, $userNo, $dbInfo['msgno']);
                }
            }
        }
    }

    return;
}


/**
* Va-Database.inc.php :: GetTotalMessageCount()
*
* This function will return the total number of database entries.
*
* @return
*/
function GetTotalMessageCount() {
    global $VA_SYSPATHS;                    // Include the global variable.

    $file = $VA_SYSPATHS['data'] . '/dbseq.dat';
    if (!is_file($file)) {
        return 0;
    }

    $fileString = ReadCfgFile($file);       // Read the file.
    return GetLongInt($fileString, 1);      // Return the total message count.
}


/**
* Va-Database.inc.php :: WriteTotalMessageCount()
*
* This function will add one to the total message count and return that integer.
*
* @return
*/
function WriteTotalMessageCount() {
    global $VA_SYSPATHS;                    // Include the global variable.

    $file = $VA_SYSPATHS['data'] . '/dbseq.dat';
    if (!is_file($file)) {
        $fileString = '0';
        $fileString = PutLongInt($fileString, 1, 1);
        $status = WriteCfgFile($file, $fileString);
        if (!$status) {
            return $status;
        }
        return 1;
    }

    $fileString = ReadCfgFile($file);       // Read the file.
    $msgcnt = GetLongInt($fileString, 1);   // Get the total message count.

    $fileString = PutLongInt($fileString, $msgcnt + 1, 1);
    $status = WriteCfgFile($file, $fileString);
    if (!$status) {
        return $status;
    }
    
    return $msgcnt;                         // Return the original value.
}


/**
* Va-Database.inc.php :: FindMessageInfo()
*
* This function will retrieve the variables for the specified message using the
* passed fileString. This function is identical to the GetMessageInfo function
* except you must pass the fileString. This is useful if you are going to be
* searching the same message database multiple times.
*
* @param [type] $dbInfo
* @param [type] $msgNo
* @param [type] $fileString
* @return
*/
function FindMessageInfo($dbInfo, $msgNo, $fileString) {
    if (!is_array($dbInfo)) {
        trigger_error(VA_Error(1));
    }
    if ((!is_int($msgNo)) && (!is_string($msgNo))) {
        trigger_error(VA_Error(1));
    }
    if (!is_string($fileString)) {
        trigger_error(VA_Error(1));
    }
    $msgNo = intval($msgNo);
    if ($msgNo < 1) {
        $msgno = 1;
    }
    if ($msgNo > $dbInfo['msgno']) {
        return FALSE;
    }

    global $VA_SYSPATHS;                    // Include global variable.

    $msgInfo = array();
    $offset  = 582 * ($msgNo - 1);
    $msgInfo['topicno']     = $dbInfo['topicno'];
    $msgInfo['topic']       = $dbInfo['topic'];
    $msgInfo['dbno']        = $dbInfo['dbno'];
    $msgInfo['fromuserno']  = GetLongInt($fileString, 1 + $offset);
    $msgInfo['touserno']    = GetLongInt($fileString, 5 + $offset);
    $msgInfo['fromhandle']  = GetString($fileString, 9 + $offset, 40);
    if ($msgInfo['fromhandle'] == '') {
        $msgInfo['fromhandle'] = 'Unknown';
    }
    $msgInfo['tohandle']    = GetString($fileString, 49 + $offset, 40);
    if ($msgInfo['tohandle'] == '') {
        $msgInfo['tohandle'] = 'All';
    }
    $msgInfo['fromnetnode'] = GetString($fileString, 89 + $offset, 80);
    $msgInfo['tonetnode']   = GetString($fileString, 169 + $offset, 80);
    $msgInfo['fromnetid']   = GetInteger($fileString, 249 + $offset);
    $msgInfo['tonetid']     = GetInteger($fileString, 251 + $offset);
    $msgInfo['timestamp']   = GetLongInt($fileString, 253 + $offset) - (date("O") / 100 * 60 * 60);
    $msgInfo['daysold']     = intval((time() - $msgInfo['timestamp']) / (60 * 60 * 24));
    $msgInfo['sysmsgno']    = GetLongInt($fileString, 257 + $offset);
    $msgInfo['subject']     = GetString($fileString, 269 + $offset, 64);
    $msgInfo['deleted']     = GetInteger($fileString, 333 + $offset);
    $msgInfo['creation']    = GetString($fileString, 335 + $offset, 38);
    $msgInfo['date']        = GetString($fileString, 335 + $offset, 17);
    $msgInfo['time']        = GetString($fileString, 352 + $offset, 9);
    $msgInfo['timezone']    = GetString($fileString, 361 + $offset, 12);
    $msgInfo['offline']     = GetInteger($fileString, 375 + $offset);
    $msgInfo['attfilesize'] = GetLongInt($fileString, 379 + $offset);
    $msgInfo['attfilename'] = GetString($fileString, 383 + $offset, 12);
    $msgInfo['attfilepath'] = GetString($fileString,395 + $offset, 64);
    $msgInfo['downloadno']  = GetInteger($fileString, 463 + $offset);
    $msgInfo['filename']    = GetString($fileString, 471 + $offset, 8);
    $msgInfo['msgpos']      = GetLongInt($fileString, 479 + $offset);
    $msgInfo['msglength']   = GetLongInt($fileString, 483 + $offset);
    $msgInfo['originnet']   = GetLongInt($fileString, 487 + $offset);
    $msgInfo['originnode']  = GetLongInt($fileString, 491 + $offset);

    return $msgInfo;                        // Return the message information.
}


/**
* Va-Database.inc.php :: GetMessageInfo()
*
* This function will retrieve the variables for the specified message.
*
* @param [type] $dbInfo
* @param [type] $msgNo
* @return
*/
function GetMessageInfo($dbInfo, $msgNo) {
    if (!is_array($dbInfo)) {
        trigger_error(VA_Error(1));
    }
    if ((!is_int($msgNo)) && (!is_string($msgNo))) {
        trigger_error(VA_Error(1));
    }
    $msgNo = intval($msgNo);
    if ($msgNo < 1) {
        $msgno = 1;
    }
    if ($msgNo > $dbInfo['msgno']) {
        return FALSE;
    }

    global $VA_SYSPATHS;                    // Include global variable.

    // Set the filename.
    $file    = $VA_SYSPATHS['db'] . '/' . $dbInfo['filename'];

    $datFile = $file . '.DAT';              // Set the database filename.
    if (!is_file($datFile)) {
        return FALSE;
    }

    $fileString = ReadCfgFile($datFile);    // Read the file.

    $msgInfo = array();
    $offset  = 582 * ($msgNo - 1);
    $msgInfo['topicno']     = $dbInfo['topicno'];
    $msgInfo['topic']       = $dbInfo['topic'];
    $msgInfo['dbno']        = $dbInfo['dbno'];
    $msgInfo['fromuserno']  = GetLongInt($fileString, 1 + $offset);
    $msgInfo['touserno']    = GetLongInt($fileString, 5 + $offset);
    $msgInfo['fromhandle']  = GetString($fileString, 9 + $offset, 40);
    if ($msgInfo['fromhandle'] == '') {
        $msgInfo['fromhandle'] = 'Unknown';
    }
    $msgInfo['tohandle']    = GetString($fileString, 49 + $offset, 40);
    if ($msgInfo['tohandle'] == '') {
        $msgInfo['tohandle'] = 'All';
    }
    $msgInfo['fromnetnode'] = GetString($fileString, 89 + $offset, 80);
    $msgInfo['tonetnode']   = GetString($fileString, 169 + $offset, 80);
    $msgInfo['fromnetid']   = GetInteger($fileString, 249 + $offset);
    $msgInfo['tonetid']     = GetInteger($fileString, 251 + $offset);
    $msgInfo['timestamp']   = GetLongInt($fileString, 253 + $offset) - (date("O") / 100 * 60 * 60);
    $msgInfo['daysold']     = intval((time() - $msgInfo['timestamp']) / (60 * 60 * 24));
    $msgInfo['sysmsgno']    = GetLongInt($fileString, 257 + $offset);
    $msgInfo['subject']     = GetString($fileString, 269 + $offset, 64);
    $msgInfo['deleted']     = GetInteger($fileString, 333 + $offset);
    $msgInfo['creation']    = GetString($fileString, 335 + $offset, 38);
    $msgInfo['date']        = GetString($fileString, 335 + $offset, 17);
    $msgInfo['time']        = GetString($fileString, 352 + $offset, 9);
    $msgInfo['timezone']    = GetString($fileString, 361 + $offset, 12);
    $msgInfo['offline']     = GetInteger($fileString, 375 + $offset);
    $msgInfo['attfilesize'] = GetLongInt($fileString, 379 + $offset);
    $msgInfo['attfilename'] = GetString($fileString, 383 + $offset, 12);
    $msgInfo['attfilepath'] = GetString($fileString,395 + $offset, 64);
    $msgInfo['downloadno']  = GetInteger($fileString, 463 + $offset);
    $msgInfo['filename']    = GetString($fileString, 471 + $offset, 8);
    $msgInfo['msgpos']      = GetLongInt($fileString, 479 + $offset);
    $msgInfo['msglength']   = GetLongInt($fileString, 483 + $offset);
    $msgInfo['originnet']   = GetLongInt($fileString, 487 + $offset);
    $msgInfo['originnode']  = GetLongInt($fileString, 491 + $offset);

    return $msgInfo;                        // Return the message information.
}


/**
* Va-Database.inc.php :: WriteMessageInfo()
*
* This function will allow you to write new message information into the
* corresponding .DAT file. You cannot change the creation date.
*
* @param [type] $dbInfo
* @param [type] $msgNo
* @param [type] $configVariable
* @return
*/
function WriteMessageInfo($dbInfo, $msgNo, $configVariable) {
    if (!is_array($dbInfo)) {
        trigger_error(VA_Error(1));
    }
    if ((!is_int($msgNo)) && (!is_string($msgNo))) {
        trigger_error(VA_Error(1));
    }
    if (!is_array($configVariable)) {
        trigger_error(VA_Error(1));
    }
    $msgNo = intval($msgNo);
    if ($msgNo < 1) {
        return FALSE;
    }
    if ($msgNo > $dbInfo['msgno']) {
        return FALSE;
    }


    global $VA_SYSPATHS;                    // Include global variable.

    // Set the filename.
    $file    = $VA_SYSPATHS['db'] . '/' . $dbInfo['filename'];
    $datFile = $file . '.DAT';              // Set the database filename.
    if (!is_file($datFile)) {
        return FALSE;
    }

    $fileString = ReadCfgFile($datFile);    // Read the file.

    $offset = 582 * ($msgNo - 1);
    if (isset($configVariable['fromuserno']))  $fileString = PutLongInt($fileString, $configVariable['fromuserno'], 1 + $offset);
    if (isset($configVariable['touserno']))    $fileString = PutLongInt($fileString, $configVariable['touserno'], 5 + $offset);
    if (isset($configVariable['fromhandle']))  $fileString = PutString($fileString, $configVariable['fromhandle'], 9 + $offset, 40);
    if (isset($configVariable['tohandle']))    $fileString = PutString($fileString, $configVariable['tohandle'], 49 + $offset, 40);
    if (isset($configVariable['fromnetnode'])) $fileString = PutString($fileString, $configVariable['fromnetnode'], 89 + $offset, 80);
    if (isset($configVariable['tonetnode']))   $fileString = PutString($fileString, $configVariable['tonetnode'], 169 + $offset, 80);
    if (isset($configVariable['fromnetid']))   $fileString = PutInteger($fileString, $configVariable['fromnetid'], 249 + $offset);
    if (isset($configVariable['tonetid']))     $fileString = PutInteger($fileString, $configVariable['tonetid'], 251 + $offset);
    if (isset($configVariable['sysmsgno']))    $fileString = PutLongInt($fileString, $configVariable['sysmsgno'], 257 + $offset);
    if (isset($configVariable['subject']))     $fileString = PutString($fileString, $configVariable['subject'], 269 + $offset, 64);
    if (isset($configVariable['deleted']))     $fileString = PutInteger($fileString, $configVariable['deleted'], 333 + $offset);
    if (isset($configVariable['offline']))     $fileString = PutInteger($fileString, $configVariable['offline'], 375 + $offset);
    if (isset($configVariable['attfilesize'])) $fileString = PutLongInt($fileString, $configVariable['attfilesize'], 379 + $offset);
    if (isset($configVariable['attfilename'])) $fileString = PutString($fileString, $configVariable['attfilename'], 383 + $offset, 12);
    if (isset($configVariable['attfilepath'])) $fileString = PutString($fileString, $configVariable['attfilepath'], 395 + $offset, 64);
    if (isset($configVariable['downloadno']))  $fileString = PutInteger($fileString, $configVariable['downloadno'], 463 + $offset);
    if (isset($configVariable['filename']))    $fileString = PutString($fileString, $configVariable['filename'], 471 + $offset, 8);
    if (isset($configVariable['msgpos']))      $fileString = PutLongInt($fileString, $configVariable['msgpos'], 479 + $offset);
    if (isset($configVariable['msglength']))   $fileString = PutLongInt($fileString, $configVariable['msglength'], 483 + $offset);

    $status = WriteCfgFile($datFile, $fileString);

    return $status;                         // Return the result status.

}


/**
* Va-Database.inc.php :: GetMessage()
*
* This function will retrieve the message for the selected information. You may
* set $format to 1 if you would like to strip html tags or 2 for no formatting.
*
* @param [type] $dbInfo
* @param [type] $msgNo
* @param integer $format
* @return
*/
function GetMessage($dbInfo, $msgNo, $format = 0) {
    if (!is_array($dbInfo)) {
        trigger_error(VA_Error(1));
    }
    if ((!is_int($msgNo)) && (!is_string($msgNo))) {
        trigger_error(VA_Error(1));
    }
    $msgNo = intval($msgNo);
    if ($msgNo < 1) {
        $msgNo = 1;
    }
    if ($msgNo > $dbInfo['msgno']) {
        return FALSE;
    }
    if (intval($format) > 2) {
        $format = 1;
    }

    global $VA_SYSPATHS;                    // Include the global variable.

    // Set the filename.
    $file    = $VA_SYSPATHS['db'] . '/' . $dbInfo['filename'];
    $binFile = $file . '.BIN';
    if (!is_file($binFile)) {
        return FALSE;
    }

    $msgInfo = GetMessageInfo($dbInfo, $msgNo);
    
    $fp = fopen($binFile, 'r');                         // Open the file.
    fseek($fp, $msgInfo['msgpos'] - 1);
    if ($msgInfo['msglength'] > 0) {                    // If the message length is greater than zero...
        $message = fread($fp, $msgInfo['msglength']);   // get the message, otherwise...
    } else {
        $message = '';                                  // the message is nothing.
    }
    fclose($fp);                                        // Close the file.

    if ($format == 1) {                             // HTML formatting...
        $message = rtrim(preg_replace("/(=20)||(=09)/", '', strip_tags($message)));
    } else if ($format == 2) {
        $message = rtrim($message);
    } else {
        $message = rtrim(str_replace('&amp;', '&', htmlspecialchars($message)));
    }

    while ((substr($message, 0, 1) == "\n") || (substr($message, 0, 1) == "\r")) {
        $message = substr($message, 1);
    }

    // Detect MIME header and only display text version.
    $pos  = strpos(ltrim($message), "\r\n");
    $line = substr(ltrim($message), 0, $pos);
    if (is_int(strpos($line, 'MIME'))) {
        $tempMessage = ltrim(substr(ltrim($message), $pos + 1));
    } else {
        $tempMessage = ltrim($message);
    }
    $line = substr($tempMessage, 0, 2);
    if ($line == '--') {
        $pos  = strpos($tempMessage, "\r\n");
        $line = ltrim(substr($tempMessage, $pos + 1));
        if (strtolower(substr($line, 0, 8)) == 'content-') {
            $mimehdr = ltrim(substr($tempMessage, 0, $pos));
            $pos = strpos($tempMessage, "\r\n\r\n");
            $message = ltrim(substr($tempMessage, $pos + 1));
            $pos = strpos($message, $mimehdr);
            $message = trim(substr($message, 0, $pos));
        }
    }

    return $message;                        // Return the message.
}


/**
* Va-Database.inc.php :: DeleteMessage()
*
* This function will permanently delete the given message.
*
* @param [type] $dbInfo
* @param [type] $msgNo
* @return
*/
function DeleteMessage($dbInfo, $msgNo) {
    if (!is_array($dbInfo)) {
        trigger_error(VA_Error(1));
    }
    if ((!is_int($msgNo)) && (!is_string($msgNo))) {
        trigger_error(VA_Error(1));
    }
    $msgNo = intval($msgNo);
    
    global $VA_SYSPATHS;                    // Include global variable.
    $file = $VA_SYSPATHS['db'] . '/' . $dbInfo['filename'];
    $datFile = $file . '.DAT';
    $binFile = $file . '.BIN';
    
    $fileString = ReadCfgFile($datFile);    // Read the file string.
    $msgInfo = FindMessageInfo($dbInfo, $msgNo, $fileString);
    
    // Remove the message from the .BIN file.
    $string = ReadCfgFile($binFile);
    $string = substr_replace($string, '', $msgInfo['msgpos'] - 1, $msgInfo['msglength']);
    WriteCfgFile($binFile, $string);

    // Update all messages with new position information in the .DAT file.
    for ($x = $msgNo + 1; $x < $dbInfo['msgno']; $x++) {
        $info = FindMessageInfo($dbInfo, $x, $fileString);
        $info['msgpos'] = $info['msgpos'] - $msgInfo['msglength'];
        $fileString = substr_replace($fileString, MakeLongInt($info['msgpos']), (($x - 1) * 582) + 479 - 1, 4);
    }

    // Remove the message information from the .DAT file.
    $fileString = substr_replace($fileString, '', (($msgNo - 1) * 582) - 1, 582);
    $status = WriteCfgFile($datFile, $fileString);
    
    return $status;                         // Return the status.
}


/**
* Va-Database.inc.php :: WriteMessage()
*
* This function will write the message header information and the message text
* to the proper files based on the passed database information.
*
* This function is to write only new message area messages. For emails or files,
* use the appropriate function. If this is a reply to a message, set $reply to
* the message number of the original message.
*
* @param [type] $dbInfo
* @param [type] $fromUserNo
* @param [type] $msgSubject
* @param [type] $msgBody
* @param integer $reply
* @return
*/
function WriteMessage($dbInfo, $fromUserNo, $msgSubject, $msgBody, $reply = 0) {
    if (!is_array($dbInfo)) {
        trigger_error(VA_Error(1));
    }
    if ((!is_int($fromUserNo)) && (!is_string($fromUserNo))) {
        trigger_error(VA_Error(1));
    }
    if (!is_string($msgSubject)) {
        trigger_error(VA_Error(1));
    }
    if (!is_string($msgBody)) {
        trigger_error(VA_Error(1));
    }
    $fromUserNo = intval($fromUserNo);
    if ($fromUserNo < 1) {
        return FALSE;
    }
    
    global $VA_SYSPATHS;                    // Include global variable.
    global $VA_MAIN;                        // Include global variable in case it is already defined.
    global $includeDir;
    include_once($includeDir . 'va-userfile.inc.php');
    include_once($includeDir . 'va-main.inc.php');
    include_once($includeDir . 'va-networks.inc.php');
    include_once($includeDir . 'va-stats.inc.php');

    $msgSubject = rtrim(strip_tags($msgSubject));
    if (strlen($msgSubject) > 64) {
        $msgSubject = substr($msgSubject, 0, 64);
    }
    $msgBody = rtrim(wordwrap($msgBody, 80, "\r\n", 1));
    if ($dbInfo['tagline'] != 0) {                                          // If tagline is set...
        $file = $VA_SYSPATHS['text'] . '/tagline.' . $dbInfo['tagline'];    // Look for the file...
        if (is_file($file)) {
            $msgBody = $msgBody . "\r\n\r\n" . ReadCfgFile($file);          // Append the file if found.
        }
    }

    // Set the filename.
    $file    = $VA_SYSPATHS['db'] . '/' . $dbInfo['filename'];
    $binFile = $file . '.BIN';              // Set the bin filename.
    $datFile = $file . '.DAT';              // Set the database filename.

    // Get the 'from' user information and update it.
    $fileString = ReadUserfile();
    $fromUser = GetUser($fromUserNo);
    $fromUser['totalposts']++;
    $fromUser['credits'] = $fromUser['credits'] + $VA_MAIN['credperpost'];  // Award credits for posting.
    $fileString = PutUser($fileString, $fromUserNo, $fromUser);
    $status = WriteUserfile($fileString);

    // Update the BBS stats.
    $fileString = ReadStats();
    $stats = GetStats();
    $stats['poststotal']++;
    $stats['poststoday']++;
    $fileString = PutStats($fileString, $stats);
    $status = WriteStats($fileString);

    if (!is_file($binFile)) {
        $msgpos = 1;
    } else {
        $msgpos = filesize($binFile) + 1;       // Get the message starting position.
    }
    $msglen = strlen($msgBody) + 2;             // Get the message length.

    // Update the .DAT file (and update dbseq.dat).
    // Add spacing of 582 bytes of null chars.
    $newString = str_pad('', 582, chr(0));  // Create the new blank string.
    $networks = GetNetworkInfo();           // Get the local network info.

    // Check for special database settings.
    if ($dbInfo['dispname'] == 1) {
        $fromHandle = $fromUser['realname'];
    } else if ($dbInfo['dispname'] == 2) {
        $fromHandle = 'Anonymous';
    } else {
        $fromHandle = $fromUser['handle'];
    }

    // Check if a reply and set the variables accordingly.
    if ($reply != 0) {
        $origMessage = GetMessageInfo($dbInfo, $reply);
        $toUserNo    = $origMessage['fromuserno'];
        $toUser      = $origMessage['fromhandle'];
        $toNetNode   = $origMessage['fromnetnode'];
        $toNetId     = $origMessage['fromnetid'];
    } else {
        $toUserNo    = 0;
        $toUser      = '';
        $toNetNode   = '';
        $toNetId     = 0;
    }
    $newString = PutLongInt($newString, $fromUserNo, 1);
    $newString = PutLongInt($newString, $toUserNo, 5);
    $newString = PutString($newString, $fromHandle, 9, 40);
    $newString = PutString($newString, $toUser, 49, 40);
    $newString = PutString($newString, $networks[1]['settings']['systemid'], 89, 80);
    $newString = PutString($newString, $toNetNode, 169, 80);
    $newString = PutInteger($newString, 1, 249);
    $newString = PutInteger($newString, $toNetId, 251);
    $newString = PutLongInt($newString, time() + (date("O") / 100 * 60 * 60), 253);
    $newString = PutLongInt($newString, WriteTotalMessageCount(), 257);
    $newString = PutString($newString, $msgSubject, 269, 64);
    $newString = PutInteger($newString, 0, 333);
    $newString = PutString($newString, date("D M d, Y H:i:s ") . $VA_MAIN['timezone'], 335, 38);
    $newString = PutInteger($newString, 0, 375);
    $newString = PutLongInt($newString, 0, 379);
    $newString = PutString($newString, '', 383, 12);
    $newString = PutString($newString, '', 395, 64);
    $newString = PutInteger($newString, 0, 463);
    $newString = PutString($newString, $dbInfo['filename'], 471, 8);
    $newString = PutLongInt($newString, $msgpos, 479);
    $newString = PutLongInt($newString, $msglen, 483);
    if ($reply != 0) {
        $newString = PutLongInt($newString, $toNetId, 487);
    }
    $newString = PutLongInt($newString, 0, 491);

    $fileString = ReadCfgFile($datFile);            // Read the file.
    $fileString = $fileString . $newString;         // Append the new data to the original file string.
    $status = WriteCfgFile($datFile, $fileString);  // Write the new data file.

    // Add the message to the .BIN file.
    $fp = fopen($binFile, 'a');                 // Open the file.
    flock($fp, LOCK_EX);                        // Lock the file.
    $status = fwrite($fp, $msgBody . "\r\n");   // Write the message.
    flock($fp, LOCK_UN);                        // Unlock the file.
    fclose($fp);                                // Close the file.

    // Create validation file or spool.net file based on db settings.
    if ($dbInfo['validate'] == 1) {
        $file = $VA_SYSPATHS['data'] . '/validate.0';   // Create local post validation file.
    } else {
        $file = $VA_SYSPATHS['data'] . '/spool.net';    // Create network spool file.
    }
    $fp = fopen($file, 'a');                                // Open the file.
    flock($fp, LOCK_EX);                                    // Lock the file.
    $status = fwrite($fp, $newString . $msgBody . "\r\n");  // Write the message.
    flock($fp, LOCK_UN);                                    // Unlock the file.
    fclose($fp);                                            // Close the file.

    return $status;                         // Return the write status.
}


/**
* Va-Database.inc.php :: WriteFile()
*
* This function will write the file header information and the file description
* to the proper files based on the passed database information. $attFile should
* be a full path to the file and the filename should be in the 8.3 format. VADV
* does not support long filenames.
*
* This function is to write only new file area entries. For emails or messages,
* use the appropriate function.
*
* @param [type] $dbInfo
* @param [type] $fromUserNo
* @param [type] $fileDesc
* @param [type] $fileExtDesc
* @param [type] $attFile
* @return
*/
function WriteFile($dbInfo, $fromUserNo, $fileDesc, $fileExtDesc, $attFile) {
    if (!is_array($dbInfo)) {
        trigger_error(VA_Error(1));
    }
    if ((!is_int($fromUserNo)) && (!is_string($fromUserNo))) {
        trigger_error(VA_Error(1));
    }
    if (!is_string($fileDesc)) {
        trigger_error(VA_Error(1));
    }
    if (!is_string($fileExtDesc)) {
        trigger_error(VA_Error(1));
    }
    $fromUserNo = intval($fromUserNo);
    if ($fromUserNo < 1) {
        return FALSE;
    }

    global $VA_SYSPATHS;                    // Include global variable.
    global $CONFIG;                         // Include VADV-PHP configuration.
    global $VA_MAIN;                        // Include global variable in case it is already defined.
    global $includeDir;
    include_once($includeDir . 'va-userfile.inc.php');
    include_once($includeDir . 'va-main.inc.php');
    include_once($includeDir . 'va-networks.inc.php');
    include_once($includeDir . 'va-stats.inc.php');

    $fileDesc = rtrim(strip_tags($fileDesc));
    if (strlen($fileDesc) > 64) {
        $fileDesc = substr($fileDesc, 0, 64);
    }
    $fileExtDesc = rtrim(wordwrap($fileExtDesc, 80, "\r\n", 1));
    if ($dbInfo['tagline'] != 0) {                                          // If tagline is set...
        $file = $VA_SYSPATHS['text'] . '/tagline.' . $dbInfo['tagline'];    // Look for the file...
        if (is_file($file)) {
            $fileExtDesc = $fileExtDesc . "\r\n\r\n" . ReadCfgFile($file);  // Append the file if found.
        }
    }

    // Setup file variables if a file was attached.
    if ((trim($attFile) != '') && (is_file($attFile))) {
        $attFilesize = filesize($attFile);
        $pos = strrpos($attFile, '/');
        if ($pos == 0) {
            $pos = strrpos($attFile, "\\");
        }
        if ($pos > 0) {
            $attFilename = substr($attFile, $pos + 1);
        } else {
            $attFilesize = 0;
            $attFilename = '';
        }
    } else {
        $attFilesize = 0;
        $attFilename = '';
    }

    // Set the filename.
    $file    = $VA_SYSPATHS['db'] . '/' . $dbInfo['filename'];
    $binFile = $file . '.BIN';              // Set the bin filename.
    $datFile = $file . '.DAT';              // Set the database filename.

    // Get the 'from' user information and update it.
    $fileString = ReadUserfile();
    $fromUser = GetUser($fromUserNo);
    $fromUser['uplfiles']++;
    $uplKbCredit = intval($attFilesize / 1024) * $VA_MAIN['credperuplk'];
    $fromUser['credits'] = $fromUser['credits'] + $uplKbCredit; // Award credits for posting.
    $fileString = PutUser($fileString, $fromUserNo, $fromUser);
    $status = WriteUserfile($fileString);

    // Update the BBS stats.
    $fileString = ReadStats();
    $stats = GetStats();
    $stats['uploadstotal']++;
    $stats['uploadstoday']++;
    $fileString = PutStats($fileString, $stats);
    $status = WriteStats($fileString);

    if (!is_file($binFile)) {
        $msgpos = 1;
    } else {
        $msgpos = filesize($binFile) + 1;       // Get the message starting position.
    }
    $msglen = strlen($fileExtDesc) + 2;         // Get the message length.

    // Update the .DAT file (and update dbseq.dat).
    // Add spacing of 582 bytes of null chars.
    $newString = str_pad('', 582, chr(0));  // Create the new blank string.
    $networks = GetNetworkInfo();           // Get the local network info.

    // Check for special database settings.
    if ($dbInfo['dispname'] == 1) {
        $fromHandle = $fromUser['realname'];
    } else if ($dbInfo['dispname'] == 2) {
        $fromHandle = 'Anonymous';
    } else {
        $fromHandle = $fromUser['handle'];
    }

    $newString = PutLongInt($newString, $fromUserNo, 1);
    $newString = PutLongInt($newString, 0, 5);
    $newString = PutString($newString, $fromHandle, 9, 40);
    $newString = PutString($newString, '', 49, 40);
    $newString = PutString($newString, $networks[1]['settings']['systemid'], 89, 80);
    $newString = PutString($newString, '', 169, 80);
    $newString = PutInteger($newString, 1, 249);
    $newString = PutInteger($newString, 0, 251);
    $newString = PutLongInt($newString, time() + (date("O") / 100 * 60 * 60), 253);
    $newString = PutLongInt($newString, WriteTotalMessageCount(), 257);
    $newString = PutString($newString, $fileDesc, 269, 64);
    $newString = PutInteger($newString, 0, 333);
    $newString = PutString($newString, date("D M d, Y H:i:s ") . $VA_MAIN['timezone'], 335, 38);
    $newString = PutInteger($newString, 0, 375);
    $newString = PutLongInt($newString, $attFilesize, 379);
    $newString = PutString($newString, strtoupper($attFilename), 383, 12);
    $newString = PutString($newString, '', 395, 64);
    $newString = PutInteger($newString, 0, 463);
    $newString = PutString($newString, $dbInfo['filename'], 471, 8);
    $newString = PutLongInt($newString, $msgpos, 479);
    $newString = PutLongInt($newString, $msglen, 483);
    $newString = PutLongInt($newString, 0, 487);
    $newString = PutLongInt($newString, 0, 491);

    $fileString = ReadCfgFile($datFile);            // Read the file.
    $fileString = $fileString . $newString;         // Append the new data to the original file string.
    $status = WriteCfgFile($datFile, $fileString);  // Write the new data file.

    // Add the message to the .BIN file.
    $fp = fopen($binFile, 'a');                 // Open the file.
    flock($fp, LOCK_EX);                        // Lock the file.
    $status = fwrite($fp, $fileExtDesc . "\r\n");   // Write the message.
    flock($fp, LOCK_UN);                        // Unlock the file.
    fclose($fp);                                // Close the file.

    return $status;                         // Return the write status.
}


/**
* Va-Database.inc.php :: WriteEmail()
*
* This function will write the email header information and the message text
* to the proper files based on the passed database information. If a file is
* attached to the email, then the filename must be in the 8.3 format. VADV does
* not support long filenames.
*
* This function is to write only new email messages. For message area or files,
* use the appropriate function. If this is a reply to a message, set $reply to
* the message number of the original message.
*
* The toUserString is in the format of user@node*netnumber, for example:
* to send an email to me, it would be 1@1*80. Local messages are given a network
* number of 0.
*
* @param [type] $fromUserNo
* @param [type] $toUserString
* @param [type] $msgSubject
* @param [type] $msgBody
* @param integer $reply
* @param string $attFile
* @param integer $oneliner
* @return
*/
function WriteEmail($fromUserNo, $toUserString, $msgSubject, $msgBody,
                    $reply = 0, $attFile = '', $oneliner = 0) {
    if ((!is_int($fromUserNo)) && (!is_string($fromUserNo))) {
        trigger_error(VA_Error(1));
    }
    if ((!is_int($toUserString)) && (!is_string($toUserString))) {
        trigger_error(VA_Error(1));
    }
    if (!is_string($msgSubject)) {
        trigger_error(VA_Error(1));
    }
    if (!is_string($msgBody)) {
        trigger_error(VA_Error(1));
    }
    $fromUserNo = intval($fromUserNo);
    if ($fromUserNo < 1) {
        return FALSE;
    }
    
    global $VA_SYSPATHS;                    // Include global variable.
    global $CONFIG;                         // Include VADV-PHP Configuration.
    global $VA_MAIN;                        // Include global variable in case it is already defined.
    global $includeDir;
    include_once($includeDir . 'va-userfile.inc.php');
    include_once($includeDir . 'va-main.inc.php');
    include_once($includeDir . 'va-networks.inc.php');
    include_once($includeDir . 'va-stats.inc.php');

    $toUserString = trim($toUserString);
    $msgSubject   = rtrim(strip_tags($msgSubject));
    if (strlen($msgSubject) > 64) {
        $msgSubject = substr($msgSubject, 0, 64);
    }
    $msgBody      = rtrim(wordwrap($msgBody, 80, "\r\n", 1));
    $file = $VA_SYSPATHS['text'] . '/email.tag';                // Check for email tagline file.
    if (is_file($file)) {
        $msgBody = $msgBody . "\r\n\r\n" . ReadCfgFile($file);  // Add the tagline if there.
    }

    $networks = GetNetworkInfo();           // Get the local network info.
    if ($oneliner == 0) {
        $dbInfo = SetEmailInfo($fromUserNo);
    } else {
        $dbInfo = SetOnelinerInfo();
    }
    
    // Setup file variables if a file was attached.
    if ((trim($attFile) != '') && (is_file($attFile))) {
        $attFilesize = filesize($attFile);
        $pos = strrpos($attFile, '/');
        if ($pos == 0) {
            $pos = strrpos($attFile, "\\");
        }
        if ($pos > 0) {
            $attFilename = substr($attFile, $pos + 1);
            if ($CONFIG['hostname'] != '') {
                $attFilepath = ConvertUnc($attFile);
            } else {
                $attFilepath = $attFile;
            }
        } else {
            $attFilesize = 0;
            $attFilename = '';
            $attFilepath = '';
        }
    } else {
        $attFilesize = 0;
        $attFilename = '';
        $attFilepath = '';
    }

    // Break down the toUserString variable.
    $atpos = strpos($toUserString, '@');
    $aspos = strpos($toUserString, '*');
    $toUserNo  = trim(substr($toUserString, 0, $atpos));
    $toNetNode = trim(substr($toUserString, $atpos + 1, $aspos - $atpos - 1));
    $toNetId   = trim(substr($toUserString, $aspos + 1));

    // Check to see if it a local net id.
    if (($toNetId == 1) && ($toNetNode == $networks[1]['settings']['systemid'])) {
        $toNetNode = 0;
        $toNetId   = 0;
    }

    // If a local email, check to see if the user exists.
    if ($toNetId == 0) {
        if (is_numeric($toUserNo)) {
            $toUser = GetUser($toUserNo);
        } else {
            $toUser = FindHandle($toUserNo);
            if ($toUser == 0) {
                $toUser = FindRealName($toUserNo);
                if ($toUser == 0) {
                    die('<br /><h4>The user specified is invalid. Please press the BACK button on your browser.</h4>');
                }
            }
            $toUserNo = $toUser;
            $toUser = GetUser($toUser);
        }
        // Check if the user has their email forwarded.
        if ($toUser['mailfwdflag'] == 1) {
            // If forwarded to another local user...
            if (($toUser['mailfwdnetno'] == 1) && ($toUser['mailfwdnode'] == $networks[1]['settings']['systemid'])) {
                $toUserNo = $toUser['mailfwduserno'];
                $toUser = GetUser($toUserNo);
            } else {
                // If forwarded to another location...
                if ($toUser['mailfwduserno'] == 0) {    // If the to user no field is blank...
                    $toUserNo = $toUser['mailfwdhandle'];
                } else {                                // If the to user no field is not blank...
                    $toUserNo = $toUser['mailfwduserno'];
                }
                $toNetNode = $toUser['mailfwdnode'];
                $toNetId   = $toUser['mailfwdnetno'];
            }
        }
    }

    // Set the toUserHandle variable.
    if ($toNetId == 0) {
        $toUserHandle = $toUser['handle'];
        $toNetNode    = $networks[1]['settings']['systemid'];
    } else {
        if (is_numeric($toUserNo)) {
            $toUserHandle = '';
        } else {
            $toUserHandle = $toUserNo;
            $toUserNo = 0;
        }
    }

    // Set the filename.
    $file    = $VA_SYSPATHS['db'] . '/' . $dbInfo['filename'];
    $binFile = $file . '.BIN';              // Set the bin filename.
    $datFile = $file . '.DAT';              // Set the database filename.

    // Get the 'from' user information and update it.
    $fileString = ReadUserfile();
    $fromUser = GetUser($fromUserNo);
    $fromUser['totalemail']++;
    $fileString = PutUser($fileString, $fromUserNo, $fromUser);
    $status = WriteUserfile($fileString);

    // Update the BBS stats.
    $fileString = ReadStats();
    $stats = GetStats();
    $stats['emailstotal']++;
    $stats['emailstoday']++;
    $fileString = PutStats($fileString, $stats);
    $status = WriteStats($fileString);

    if ($toNetId != 0) {
        $msgpos = 0;
    } else {
        if (!is_file($binFile)) {
            $msgpos = 1;
        } else {
            $msgpos = filesize($binFile) + 1;   // Get the message starting position.
        }
    }
    $msglen = strlen($msgBody) + 2;             // Get the message length.

    // Add spacing of 582 bytes of null chars.
    $newString = str_pad('', 582, chr(0));      // Create the new blank string.

    $newString = PutLongInt($newString, $fromUserNo, 1);
    $newString = PutLongInt($newString, $toUserNo, 5);
    $newString = PutString($newString, $fromUser['handle'], 9, 40);
    $newString = PutString($newString, $toUserHandle, 49, 40);
    $newString = PutString($newString, $networks[1]['settings']['systemid'], 89, 80);
    $newString = PutString($newString, $toNetNode, 169, 80);
    $newString = PutInteger($newString, 1, 249);
    if ($toNetId == 0) {
        $newString = PutInteger($newString, 1, 251);
    } else {
        $newString = PutInteger($newString, $toNetId, 251);
    }
    $newString = PutLongInt($newString, time() + (date("O") / 100 * 60 * 60), 253);
    $newString = PutLongInt($newString, WriteTotalMessageCount(), 257);
    $newString = PutString($newString, $msgSubject, 269, 64);
    $newString = PutInteger($newString, 0, 333);
    $newString = PutString($newString, date("D M d, Y H:i:s ") . $VA_MAIN['timezone'], 335, 38);
    $newString = PutInteger($newString, 0, 375);
    $newString = PutLongInt($newString, $attFilesize, 379);
    $newString = PutString($newString, strtoupper($attFilename), 383, 12);
    $newString = PutString($newString, strtoupper($attFilepath), 395, 64);
    $newString = PutInteger($newString, 0, 463);
    $newString = PutString($newString, $dbInfo['filename'], 471, 8);
    $newString = PutLongInt($newString, $msgpos, 479);
    $newString = PutLongInt($newString, $msglen, 483);
    $newString = PutLongInt($newString, $toNetId, 487);
    $newString = PutLongInt($newString, 0, 491);

    // If a local message, write it to the database, if not, write it to spool.net.
    if ($toNetId == 0) {
        // Update the .DAT file (and update dbseq.dat).
        $fileString = ReadCfgFile($datFile);            // Read the file.
        $fileString = $fileString . $newString;         // Append the new data to the original file string.
        $status = WriteCfgFile($datFile, $fileString);  // Write the new data file.

        // Add the message to the .BIN file.
        $fp = fopen($binFile, 'a');                 // Open the file.
        flock($fp, LOCK_EX);                        // Lock the file.
        $status = fwrite($fp, $msgBody . "\r\n");   // Write the message.
        flock($fp, LOCK_UN);                        // Unlock the file.
        fclose($fp);                                // Close the file.
        
        // Send a one liner to the user if online.
        $oneliner = $fromUser['handle'] . ' just sent you some email!';
        SendOneLiner($oneliner, $toUserNo);
    } else {
        // Create validation file or spool.net file based on db settings.
        $file = $VA_SYSPATHS['data'] . '/spool.net';            // Create network spool file.
        $fp = fopen($file, 'a');                                // Open the file.
        flock($fp, LOCK_EX);                                    // Lock the file.
        $status = fwrite($fp, $newString . $msgBody . "\r\n");  // Write the message.
        flock($fp, LOCK_UN);                                    // Unlock the file.
        fclose($fp);                                            // Close the file.
    }

    return $status;                         // Return the write status.
}


/**
* Va-Database.inc.php :: WriteOneliner()
*
* This function will create a oneliner message to the specified user. The message
* can be a maximum of 64 characters.
*
* @param [type] $toUserNo
* @param [type] $message
* @return
*/
function WriteOneliner($toUserNo, $message) {
    if ((!is_int($toUserNo)) && (!is_string($toUserNo))) {
        trigger_error(VA_Error(1));
    }
    if (!is_string($message)) {
        trigger_error(VA_Error(1));
    }
    $toUserNo = intval($toUserNo);
    
    global $VA_SYSPATHS;                    // Include global variable.
    global $VA_MAIN;                        // Include global variable in case it is already defined.
    global $includeDir;
    include_once($includeDir . 'va-userfile.inc.php');
    include_once($includeDir . 'va-main.inc.php');
    include_once($includeDir . 'va-networks.inc.php');

    $dbInfo   = SetOnelinerInfo();
    $toUser   = GetUser($toUserNo);
    $networks = GetVnetInfo();
    
    $file = $VA_SYSPATHS['db'] . '/' . $dbInfo['filename'];
    $datFile = $file . '.DAT';
    $binFile = $file . '.BIN';
    
    // Write to the BIN file.
    $fileString = ReadCfgFile($binFile);
    $fileString = $fileString . "\r\n";
    WriteCfgFile($binFile, $fileString);
    
    // Write to the DAT file.
    // Add spacing of 582 bytes of null chars.
    $newString = str_pad('', 582, chr(0));  // Create the new blank string.

    $newString = PutLongInt($newString, $toUserNo, 1);
    $newString = PutLongInt($newString, $toUserNo, 5);
    $newString = PutString($newString, $toUser['handle'], 9, 40);
    $newString = PutString($newString, $toUser['handle'], 49, 40);
    $newString = PutString($newString, $networks['systemid'], 89, 80);
    $newString = PutString($newString, $networks['systemid'], 169, 80);
    $newString = PutInteger($newString, 1, 249);
    $newString = PutInteger($newString, 1, 251);
    $newString = PutLongInt($newString, time() + (date("O") / 100 * 60 * 60), 253);
    $newString = PutLongInt($newString, WriteTotalMessageCount(), 257);
    $newString = PutString($newString, $message, 269, 64);
    $newString = PutInteger($newString, 0, 333);
    $newString = PutString($newString, date("D M d, Y H:i:s ") . $VA_MAIN['timezone'], 335, 38);
    $newString = PutInteger($newString, 0, 375);
    $newString = PutLongInt($newString, 0, 379);
    $newString = PutString($newString, '', 383, 12);
    $newString = PutString($newString, '', 395, 64);
    $newString = PutInteger($newString, 0, 463);
    $newString = PutString($newString, $dbInfo['filename'], 471, 8);
    $newString = PutLongInt($newString, filesize($binFile) + 2, 479);
    $newString = PutLongInt($newString, 2, 483);
    $newString = PutLongInt($newString, 0, 487);
    $newString = PutLongInt($newString, 0, 491);

    $fileString = ReadCfgFile($datFile);
    $fileString = $fileString . $newString;
    $status = WriteCfgFile($datFile, $fileString);
    
    return $status;                         // Return status.
}


/**
* Va-Database.inc.php :: QuoteMessage()
*
* This function will take the passed message and add > to the beginning of each
* line and truncate each line to 78 characters. This is used when quoting a
* message a user is replying to.
*
* @param [type] $message
* @return
*/
function QuoteMessage($message) {
    $message = preg_replace("/\r\n|\n|\r/", "\r\n", StripBBSCodes($message));
    $message = explode("\r\n", $message);
    for ($x = 0; $x < count($message); $x++) {
        if (strlen(rtrim($message[$x])) >= 78) {
            $message[$x] = substr(rtrim($message[$x]), 0, 78);
        }
        $message[$x] = rtrim($message[$x]);
    }
    $message = "\r\n\r\n> " . implode("\r\n> ", $message) . "\r\n";
    
    return $message;
}


/**
* Va-Database.inc.php :: GetMessageCount()
*
* This function will determine the number of messages contained within the
* specified database.
*
* @param [type] $dbInfo
* @return
*/
function GetMessageCount($dbInfo) {
    if (!is_array($dbInfo)) {
        trigger_error(VA_Error(1));
    }

    global $VA_SYSPATHS;                    // Include the global variable.
    
    // Set the filename.
    $file    = $VA_SYSPATHS['db'] . '/' . $dbInfo['filename'];

    $datFile = $file . '.DAT';              // Add the datafile extension.
    if (!is_file($datFile)) {
        return FALSE;
    }

    $fileSize = filesize($datFile);         // Determine the size of the file.
    $msgNo    = intval($fileSize / 582);    // Determine the number of messages.

    return $msgNo;                          // Return the message count.
}


/**
* Va-Database.inc.php :: GetEmailCount()
*
* This function will determine the number of messages contained within the
* email database for the specified user. It returns an array containing
* the number of messages total (msgno) and the number of messages that have not
* been read (unread) and the number of deleted messages (deleted).
*
* @param [type] $userNo
* @return
*/
function GetEmailCount($userNo) {
    if (!is_int($userNo) && (!is_string($userNo))) {
        trigger_error(VA_Error(1));
    }
    $userNo = intval($userNo);

    global $VA_SYSPATHS;                    // Include the global variable.

    // Set the filename.
    $datFile    = $VA_SYSPATHS['db'] . '/email.dat';
    if (!is_file($datFile)) {
        return FALSE;
    }

    $fileString = ReadCfgFile($datFile);    // Read the file.

    $count['msgno']   = 0;                  // Set the default value.
    $count['unread']  = 0;
    $count['deleted'] = 0;
    $offset = 582;
    for ($x = 0; $offset < filesize($datFile); $x++) {
        $offset = 582 * $x;
        $user = GetLongInt($fileString, 5 + $offset);
        if ($user == $userNo) {
            $delFlag = GetInteger($fileString, 333 + $offset);
            if ($delFlag == 1) {
                $count['deleted']++;
            }
            $readFlag = GetInteger($fileString, 375 + $offset);
            if ($readFlag == 0) {
                $count['unread']++;
            }
            $count['msgno']++;
        }
    }

    return $count;                          // Return the message count.
}


/**
* Va-Database.inc.php :: GetMessagesTo()
*
* This function will scan the current database for all messages to the current
* user and store them into an array.
*
* @param [type] $dbInfo
* @param [type] $userNo
* @return
*/
function SearchMessagesTo($dbInfo, $userNo) {
    if (!is_array($dbInfo)) {
        trigger_error(VA_Error(1));
    }
    if ((!is_int($userNo)) && (!is_string($userNo))) {
        trigger_error(VA_Error(1));
    }
    $userNo = intval($userNo);
    
    if ($userNo == 0) {
        $msgTo[1] = '';
        return $msgTo;
    }

    global $includeDir;
    include_once($includeDir . 'va-networks.inc.php');  // Include network information.

    // This function has been optimized for speed and does not use the pre-
    // written functions above.
    global $VA_SYSPATHS;                    // Include global variable.

    // Set the filename.
    $file    = $VA_SYSPATHS['db'] . '/' . $dbInfo['filename'];

    $datFile = $file . '.DAT';              // Set the database filename.
    if (!is_file($datFile)) {
        return FALSE;
    }

    $netInfo    = GetNetworkInfo();
    $fileString = ReadCfgFile($datFile);    // Read the database file.

    $cnt     = 1;                           // Counter.
    $msgTo   = array();

    for ($msgNo = 1; $msgNo <= $dbInfo['msgno']; $msgNo++) {
        $offset = 582 * ($msgNo - 1);
        $msgInfo['touserno']  = GetLongInt($fileString, 5 + $offset);
        if ($msgInfo['touserno'] == $userNo) {
            $msgInfo['tonetid'] = GetInteger($fileString, 251 + $offset);
            if ((isset($netInfo[$msgInfo['tonetid']]['netno'])) &&
               ($msgInfo['tonetid'] == $netInfo[$msgInfo['tonetid']]['netno'])) {
                $msgInfo['tonetnode'] = GetString($fileString, 169 + $offset, 80);
                if ($msgInfo['tonetnode'] == $netInfo[$msgInfo['tonetid']]['settings']['systemid']) {
                    $msgTo[$cnt] = $msgNo;  // Add the message number to the array.
                    $cnt++;                 // Increase the counter by one.
                }
            }
        }
    }

    $msgTo[0] = $cnt - 1;                   // Set the total number of messages.

    return $msgTo;                          // Return the messages from this user.
}


/**
* Va-Database.inc.php :: GetMessagesFrom()
*
* This function will scan the current database for all messages from the
* current user and store them into an array.
*
* @param [type] $dbInfo
* @param [type] $userNo
* @return
*/
function SearchMessagesFrom($dbInfo, $userNo) {
    if (!is_array($dbInfo)) {
        trigger_error(VA_Error(1));
    }
    if ((!is_int($userNo)) && (!is_string($userNo))) {
        trigger_error(VA_Error(1));
    }
    $userNo = intval($userNo);

    if ($userNo == 0) {
        $msgFrom[1] = '';
        return $msgFrom;
    }

    global $includeDir;
    include_once($includeDir . 'va-networks.inc.php');  // Include network information.

    // This function has been optimized for speed and does not use the pre-
    // written functions above.
    global $VA_SYSPATHS;                    // Include global variable.

    // Set the filename.
    $file    = $VA_SYSPATHS['db'] . '/' . $dbInfo['filename'];
    $datFile = $file . '.DAT';              // Set the database filename.

    if (!is_file($datFile)) {
        return FALSE;
    }

    $netInfo    = GetNetworkInfo();
    $fileString = ReadCfgFile($datFile);    // Read the database file.

    $cnt     = 1;                           // Counter.
    $msgFrom = array();

    for ($msgNo = 1; $msgNo <= $dbInfo['msgno']; $msgNo++) {
        $offset = 582 * ($msgNo - 1);
        $msgInfo['fromuserno']  = GetLongInt($fileString, 1 + $offset);
        if ($msgInfo['fromuserno'] == $userNo) {
            $msgInfo['fromnetid'] = GetInteger($fileString, 249 + $offset);
            if ((isset($netInfo[$msgInfo['fromnetid']]['netno'])) &&
               ($msgInfo['fromnetid'] == $netInfo[$msgInfo['fromnetid']]['netno'])) {
                $msgInfo['fromnetnode'] = GetString($fileString, 89 + $offset, 80);
                if ($msgInfo['fromnetnode'] == $netInfo[$msgInfo['fromnetid']]['settings']['systemid']) {
                    $msgFrom[$cnt] = $msgNo;    // Add the message number to the array.
                    $cnt++;                     // Increase the counter by one.
                }
            }
        }
    }

    $msgFrom[0] = $cnt - 1;                 // Set the total number of messages.

    return $msgFrom;                        // Return the messages from this user.
}


/**
* Va-Database.inc.php :: SetOnelinerInfo()
*
* This function will return the oneliner database information in the same format
* as dbInfo.
*
* @return
*/
function SetOnelinerInfo() {
    $onelinerInfo['topicno']    = '';
    $onelinerInfo['topic']      = 'Oneliners';
    $onelinerInfo['dbno']       = 0;
    $onelinerInfo['filename']   = 'ONELINE';
    $onelinerInfo['filepath']   = '';
    $onelinerInfo['desc']       = 'Oneliners';
    $onelinerInfo['private']    = 0;
    $onelinerInfo['dbgroup']    = '';
    $onelinerInfo['maxmsg']     = 0;
    $onelinerInfo['minreadsl']  = 0;
    $onelinerInfo['minwritesl'] = 0;
    $onelinerInfo['agelimit']   = 0;
    $onelinerInfo['accflags']   = 0;
    $onelinerInfo['tagline']    = 0;
    $onelinerInfo['dispname']   = 0;
    $onelinerInfo['randtitle']  = 0;
    $onelinerInfo['qwkconfno']  = 0;
    $onelinerInfo['moderator']  = 0;
    $onelinerInfo['validate']   = 0;
    $onelinerInfo['freqable']   = 0;
    $onelinerInfo['devicetype'] = 0;
    $onelinerInfo['dbtype']     = 'E';
    $onelinerInfo['msgno']      = GetMessageCount($onelinerInfo);

    return $onelinerInfo;                   // Return the oneliner database information.
}


/**
* Va-Database.inc.php :: SetUploadInfo()
*
* This function will return the upload database information in the same format
* as dbInfo.
*
* @return
*/
function SetUploadInfo() {
    global $CONFIG;
    global $VA_SYSPATHS;

    $uploadsInfo['topicno']    = '';
    $uploadsInfo['topic']      = 'Uploads';
    $uploadsInfo['dbno']       = 0;
    $uploadsInfo['filename']   = 'UPLOADS';
    if ($CONFIG['filehost'] != '') {
        $emailInfo['filepath'] = ConvertUnc($VA_SYSPATHS['sysop']);
    } else {
        $emailInfo['filepath'] = $VA_SYSPATHS['sysop'];
    }
    $uploadsInfo['desc']       = 'Temporary Upload DB';
    $uploadsInfo['private']    = 0;
    $uploadsInfo['dbgroup']    = '';
    $uploadsInfo['maxmsg']     = 0;
    $uploadsInfo['minreadsl']  = 250;
    $uploadsInfo['minwritesl'] = 0;
    $uploadsInfo['agelimit']   = 0;
    $uploadsInfo['accflags']   = 0;
    $uploadsInfo['tagline']    = 0;
    $uploadsInfo['dispname']   = 0;
    $uploadsInfo['randtitle']  = 0;
    $uploadsInfo['qwkconfno']  = 0;
    $uploadsInfo['moderator']  = 0;
    $uploadsInfo['validate']   = 0;
    $uploadsInfo['freqable']   = 0;
    $uploadsInfo['devicetype'] = 0;
    $uploadsInfo['dbtype']     = 'F';
    $uploadsInfo['msgno']      = GetMessageCount($uploadsInfo);

    return $uploadsInfo;                   // Return the oneliner database information.
}


/**
* Va-Database.inc.php :: SetEmailInfo()
*
* This function will return the email database information in the same format
* as dbInfo.
*
* @param [type] $userNo
* @return
*/
function SetEmailInfo($userNo) {
    if ((!is_int($userNo)) && (!is_string($userNo))) {
        trigger_error(VA_Error(1));
    }
    $userNo = intval($userNo);
    
    global $CONFIG;
    global $VA_SYSPATHS;
    global $VA_MAIN;
    global $includeDir;
    include_once($includeDir . 'va-main.inc.php');

    $emailInfo['topicno']    = '';
    $emailInfo['topic']      = 'Email';
    $emailInfo['dbno']       = 0;
    $emailInfo['filename']   = 'EMAIL';
    if ($CONFIG['filehost'] != '') {
        $emailInfo['filepath'] = ConvertUnc($VA_SYSPATHS['sysop']);
    } else {
        $emailInfo['filepath'] = $VA_SYSPATHS['sysop'];
    }
    $emailInfo['desc']       = 'Personal Email';
    $emailInfo['private']    = 0;
    $emailInfo['dbgroup']    = '';
    $emailInfo['maxmsg']     = 0;
    $emailInfo['minreadsl']  = 0;
    $emailInfo['minwritesl'] = $VA_MAIN['emlothers'];
    $emailInfo['agelimit']   = 0;
    $emailInfo['accflags']   = 0;
    $emailInfo['tagline']    = 0;
    $emailInfo['dispname']   = 0;
    $emailInfo['randtitle']  = 0;
    $emailInfo['qwkconfno']  = 0;
    $emailInfo['moderator']  = 0;
    $emailInfo['validate']   = 0;
    $emailInfo['freqable']   = 0;
    $emailInfo['devicetype'] = 0;
    $emailInfo['dbtype']     = 'E';
    $emailInfo['msgno']      = GetMessageCount($emailInfo);
    $emailInfo['msgcnt']     = GetEmailCount($userNo);

    return $emailInfo;                      // Return the email database information.
}


/**
* Va-Database.inc.php :: GetLastPostInfo()
*
* This function will get the last post's message information of the
* specified database.
*
* @param [type] $dbInfo
* @return
*/
function GetLastPostInfo($dbInfo) {
    if (!is_array($dbInfo)) {
        trigger_error(VA_Error(1));
    }

    $lastPost = GetMessageInfo($dbInfo, $dbInfo['msgno']);

    return $lastPost;                       // Return the lastpost information.
}


/**
* Va-Database.inc.php :: DbAccess()
*
* This function will determine if the user has access clearance for the specified
* database.
*
* @param [type] $dbInfo
* @param string $userNo
* @param string $userInfo
* @return
*/
function DbAccess($dbInfo, $userNo = '', $userInfo = '') {
    if (!is_array($dbInfo)) {
        trigger_error(VA_Error(1));
    }
    if ((!is_int($userNo)) && (!is_string($userNo))) {
        trigger_error(VA_Error(1));
    }
    $userNo = intval($userNo);
    
    if (!is_array($userInfo)) {
        if ($userNo == '') {
            global $USER;
            $user = $USER;
        } else {
            global $VA_SYSPATHS;            // Include the global variable.
            global $includeDir;
            include_once($includeDir . 'va-userfile.inc.php');
            $user = GetUser($userNo);
        }
    } else {
        $user = $userInfo;
    }
    $access = FALSE;                        // Set the default access value, false.
    
    // If the sysop, give access regardless.
    if ($user['securitylevel'] >= 255) {
        return TRUE;
    }
    
    // Check security level, age and the user access flags.
    if (($user['securitylevel'] >= $dbInfo['minreadsl']) &&
        ($user['age'] >= $dbInfo['agelimit']) &&
        (($dbInfo['accflags'] == '') ||
         (strpos($user['accflags'], $dbInfo['accflags']) !== FALSE))) {
        $access = TRUE;
    }

    return $access;                         // Return the access result.
}

?>
