<?php
/*
 * ----------------------------------------------------------------------------
 * Register.inc.php                                                  06/01/2004
 * This script creates a new user account.
 * ----------------------------------------------------------------------------
 * Copyright (C) 2002-2004 Steve Winn. All Rights Reserved.
 *
 * The source code contains a fair amount of comments, which may aid anyone
 * attempting to follow the program or to modify it.
 *
 * For licensing and copyright information, view LICENSE.TXT.
 * ----------------------------------------------------------------------------
 */

require_once('include_dir.php');                    // Get include directory.
require_once($includeDir . 'va-functions.inc.php'); // A VADV-PHP script.
require_once($includeDir . 'va-database.inc.php');  // Database functions.
require_once($includeDir . 'va-main.inc.php');      // Main configuration.
require_once($includeDir . 'va-userfile.inc.php');  // User information.
require_once($includeDir . 'va-networks.inc.php');  // Network information.

if (!isset($_POST['submit'])) {
    die('<br /><h4>Invalid parameters. Please press the BACK button on your browser.</h4>');
}

if ($_POST['submit'] == 'Cancel') {
    RedirectUser('vconfig.php');
}

if ($VA_MAIN['newuserpw'] != md5(strtoupper(''))) {
    die('<br /><h4>This BBS requires a new user password. Please connect to the BBS to create a new account. Press the BACK button on your browser.</h4>');
}

if ((!isset($_POST['username'])) ||
    (!isset($_POST['realname'])) ||
    (!isset($_POST['address'])) ||
    (!isset($_POST['city'])) ||
    (!isset($_POST['state'])) ||
    (!isset($_POST['zipcode'])) ||
    (!isset($_POST['voice'])) ||
    (!isset($_POST['data'])) ||
    (!isset($_POST['password'])) ||
    (!isset($_POST['verify'])) ||
    (!isset($_POST['bdaymonth'])) ||
    (!isset($_POST['bdayday'])) ||
    (!isset($_POST['bdayyear']))) {
    die('<br /><h4>Parameters are missing. Please press the BACK button on your browser.</h4>');
}

// Format the information.
$handle    = trim($_POST['username']);
$realname  = ucwords(trim(strtolower($_POST['realname'])));
$address   = ucwords(trim(strtolower($_POST['address'])));
$city      = ucwords(trim(strtolower($_POST['city'])));
$state     = trim(strtoupper($_POST['state']));
$zipcode   = trim($_POST['zipcode']);
$voice     = trim($_POST['voice']);
$data      = trim($_POST['data']);
$password  = trim(strtoupper($_POST['password']));
$verify    = trim(strtoupper($_POST['verify']));
$bdaymonth = intval(trim($_POST['bdaymonth']));
$bdayday   = intval(trim($_POST['bdayday']));
$bdayyear  = intval(trim($_POST['bdayyear']));

// Fix the referer URL. Make sure there is no query string.
$url = parse_url($_POST['referer']);
$referer = $url['scheme'] . '://' . $url['host'] . $url['path'];
    
// Check the password.
if (($password == '') || ($password != $verify)) {
    RedirectUser($referer . '?error=password');
}

// Check the username.
if (($handle == '') || (FindHandle($handle) > 0) ||     // Check if it already exists.
    (strtolower($handle) == 'new') ||                   // Handles not allowed.
    (strtolower($handle) == '!-*<deleted>*-!') ||
    (strtolower($handle) == 'guest') ||
    (strtolower($handle) == 'sysop') ||
    (strtolower($handle) == 'postmaster') ||
    (strtolower($handle) == 'webmaster') ||
    (strtolower($handle) == 'info') ||
    (strtolower($handle) == 'help') ||
    (strtolower($handle) == 'support') ||
    (strtolower($handle) == 'bbs')) {
    RedirectUser($referer . '?error=username');
}
$file = $VA_SYSPATHS['text'] . '/trashcan.txt';
if (is_file($file)) {                       // Search for invalid words.
    $file = file($file);                    // Read the file.
    for ($x = 0; $x < count($file); $x++) {
        if (is_int(strpos(strtoupper($handle), trim(strtoupper($file[$x]))))) {
            RedirectUser($referer . '?error=username');
        }
    }
}

// Check for required information.
$required = ' ' . GetAccFlags($CONFIG['newuserreq']);   // Convert into a string.
if (is_int(strpos($required, 'A')) && ($realname == '')) {
    RedirectUser($referer . '?error=realname');
}
if (is_int(strpos($required, 'B')) && ($address == '')) {
    RedirectUser($referer . '?error=address');
}
if (is_int(strpos($required, 'C')) && ($city == '')) {
    RedirectUser($referer . '?error=city');
}
if (is_int(strpos($required, 'D')) && ($state == '')) {
    RedirectUser($referer . '?error=state');
}
if (is_int(strpos($required, 'E')) && ($zipcode == '')) {
    RedirectUser($referer . '?error=zipcode');
}
if (is_int(strpos($required, 'F')) && ($voice == '')) {
    RedirectUser($referer . '?error=voice');
}
if (is_int(strpos($required, 'G')) && ($data == '')) {
    RedirectUser($referer . '?error=data');
}
if (is_int(strpos($required, 'H')) &&
    (($bdaymonth == '') || ($bdayday == '') || ($bdayyear == ''))) {
    RedirectUser($referer . '?error=bday');
}

// Check the birthday date.
if ($bdaymonth == '') {
    $bdaymonth = date("j");                 // If nothing is supplied,
}
if ($bdayday   == '') {
    $bdayday   = date("n");                 // the user will be just born :)
}
if ($bdayyear  == '') {
    $bdayyear  = date("Y");                 // So access maybe limited.
}
if (strlen($bdayyear) == 2) {
    $bdayyear = $bdayyear + 1900;
}
if ((($bdaymonth < 1)   || ($bdaymonth > 12)) ||
    (($bdayday   < 1)    || ($bdayday   > 31)) ||
    (($bdayyear  < 1900) || ($bdayyear > date("Y")))) {
    RedirectUser($referer . '?error=bdaydate');
}

// Setup user defaults.
$newUser['userno']        = FindNewAcct();
$newUser['handle']        = $handle;
$newUser['realname']      = $realname;
$newUser['address']       = $address;
$newUser['city']          = $city;
$newUser['state']         = $state;
$newUser['zipcode']       = $zipcode;
$newUser['phone1']        = $voice;
$newUser['phone2']        = $data;
$newUser['password']      = $password;
$newUser['mailfwdflag']   = 0;
$newUser['mailfwdhandle'] = '';
$newUser['mailfwdno']     = 0;
$newUser['mailfwdnode']   = 0;
$newUser['mailfwdnetno']  = 0;
$newUser['lngdistcost']   = 0;
$newUser['lngdistacc']    = 0;
$newUser['sortfilelst']   = 0;
$newUser['pagebreak']     = 24;
$newUser['bdaymonth']     = $bdaymonth;
$newUser['bdayday']       = $bdayday;
$newUser['bdayyear']      = $bdayyear;
$newUser['accflags']      = '';
$newUser['flags']         = 'F';
$newUser['securitylevel'] = $VA_MAIN['newusersl'];
$newUser['laston']        = date("ymd");
$newUser['maxtime']       = $VA_MAIN['newusertime'];
$newUser['timecounter']   = 0;
$newUser['totalcalls']    = 0;
$newUser['totaltime']     = 0;
$newUser['videomode']     = 1;
$newUser['ratioexempt']   = 1;
$newUser['verifybday']    = 0;
$newUser['textcolor']     = 7;
$newUser['basecolor']     = 0;
$newUser['prmptcolor']    = 2;
$newUser['inputcolor']    = 5;
$newUser['boxcolor']      = 5;
$newUser['mnuselcolor']   = 1;
$newUser['headercolor']   = 2;
$newUser['extra1']        = '';
$newUser['extra2']        = '';
$newUser['extra3']        = '';
$newUser['extra4']        = '';
$newUser['extra5']        = '';
$newUser['extra6']        = '';
$newUser['extra7']        = '';
$newUser['extra8']        = '';
$newUser['macro1']        = '';
$newUser['macro2']        = '';
$newUser['uplfiles']      = 0;
$newUser['uplkb']         = 0;
$newUser['dlfiles']       = 0;
$newUser['dlkb']          = 0;
$newUser['totalposts']    = 0;
$newUser['totalemail']    = 0;
$newUser['postsread']     = 0;
$newUser['timebank']      = 0;
$newUser['credits']       = $VA_MAIN['nucredits'];
$newUser['comment']       = 'VADV-PHP';
$newUser['protocol']      = 'Z';

// Delete any mail addressed to this account.
$db     = SetEmailInfo($newUser['userno']);
$msgcnt = $db['msgcnt']['msgno'];
$file   = $VA_SYSPATHS['db'] . '/' . $db['filename'] . '.DAT';
$fileString = ReadCfgFile($file);
$pos = 0;
for ($x = 1; (($x <= $db['msgno']) && ($msgcnt > 0)); $x++) {
    $msgInfo = FindMessageInfo($db, $x, $fileString);
    if ($msgInfo['touserno'] == $newUser['userno']) {
        DeleteMessage($db, $x - $pos);
        $pos++;
    }
}

// Log the new user.
$log = 'New User Entry  "' . $newUser['handle'] . '" #' . $newUser['userno'];
WriteLog('vadv-php.log', $log);

// Write to userfile.dat.
$fileString = ReadUserfile();
$fileString = PutUser($fileString, $newUser['userno'], $newUser);
WriteUserfile($fileString);

// Write to userfile.idx.
$fileString = ReadUserfile_IDX();
$fileString = PutUser_IDX($fileString, $newUser['userno'], $newUser);
WriteUserfile_IDX($fileString);

// Write to handles.uip.
$netinfo = GetVnetInfo();
$handleuip['handle']  = str_replace(' ', '.', $newUser['handle']);
$handleuip['userno']  = $newUser['userno'];
$handleuip['netnode'] = $netinfo['systemid'];
$handleuip['netno']   = 1;
AddUserEmailInfo($newUser['userno'], $handleuip);

// Send the new user email if it exists.
$file = $VA_SYSPATHS['text'] . '/newumail.txt';
if (is_file($file)) {
    $from    = 1;
    $to      = $newUser['userno'] . '@' . $netinfo['systemid'] . '*1';
    $subject = 'Welcome New User!';
    $body    = ReadCfgFile($file);
    WriteEmail($from, $to, $subject, $body);
}

// Send an email to the sysop informing of a new user.
$from    = $newUser['userno'];
$to      = '1@' . $netinfo['systemid'] . '*1';
$subject = 'New User Alert!';
$body    = 'New BBS user: ' . $newUser['handle'] . ' (#' . $newUser['userno'] . ")\r\n"
         . 'IP Address:   ' . $_SERVER['REMOTE_ADDR'] . "\r\n"
         . 'Time & Date:  ' . date("F j, Y, g:i a") . "\r\n\r\n"
         . 'Real Name:    ' . $newUser['realname'] . "\r\n"
         . 'City, State:  ' . $newUser['city'] . ', ' . $newUser['state'] . "\r\n"
         . 'Birthday:     ' . $newUser['bdaymonth'] . '/' . $newUser['bdayday'] . '/' . $newUser['bdayyear'] . "\r\n";
WriteEmail($from, $to, $subject, $body);

// If you want a oneliner sent to the sysop as well or in the place of the email,
// uncomment the next line.
// WriteOneliner(1, $body);

// Update all new message pointers.
UpdateMessagePointers($newUser['userno'], 0);

// Now display a welcome screen and log the user in.
?>
<html>

<head>
<title>[New User Welcome] - <?php print $VA_MAIN['bbsname']; ?></title>
<meta http-equiv="Content-Type" content="text/html; charset=ISO-8559-1" />
<meta name="copyright" content="Copyright  2003, Steve Winn" />
<link rel="stylesheet" type="text/css" href="./VADVPHP.css" />
</head>

<body bgcolor="#000000">

<form name="redirect" method="post" action="login.php">
    <input type="hidden" name="referer" value="index.php">
    <input type="hidden" name="username" value="<?php print $newUser['handle']; ?>">
    <input type="hidden" name="password" value="<?php print $newUser['password']; ?>">
    <div align="center">
        <center>
        <table border="1" width="640" cellspacing="0" cellpadding="5" bgcolor="#000080" bordercolor="#FFFFFF">
            <tr>
                <td valign="top">
                    <p class="yellowTitle" align="center">Welcome New User!</p>
                    <table cellspacing="1" width="100%">
                        <tr>
                            <td valign="top">
                                <p class="navBar" align="center">You are now a registered user of this BBS!</p>
                                <p align="center" class="navBar">
                                    User #: <span class="return"><?php print $newUser['userno']; ?></span><br />
                                    Username: <span class="return"><?php print $newUser['handle']; ?></span><br />
                                    Password: <span class="return"><?php print $newUser['password']; ?></span>
                                </p>
                                <p class="infoBarBody" align="center"><input type="image" name="submit" src="images/login.gif" border="0" alt="Login!" width="54" height="25"></p>
                            </td>
                        </tr>
                    </table>
                    <p align="center"><span class="navBar">This page is powered
                    by <a class="navBar" href="http://www.vadvphp.com">VADV-PHP</a></span></p>
                    <p align="center" class="infoBarBody">If you experience any problems
                    with this website or need help, contact the
                    <a class="infoBarBody" href="mailto:<?php print $CONFIG['webmaster']; ?>">webmaster</a>.<br />
                    VADV-PHP Copyright  2002-<?php print date('Y'); ?> Aspect
                    Technologies. All Rights Reserved.</p>
                </td>
            </tr>
        </table>
        </center>
    </div>
</form>

</body>

</html>
